package data

import (
	"net/url"
	"strings"

	"github.com/pkg/errors"

	"github.com/hairyhenderson/gomplate/v3/vault"
)

func parseVaultParams(sourceURL *url.URL, args []string) (params map[string]interface{}, p string, err error) {
	p = sourceURL.Path
	params = make(map[string]interface{})
	for key, val := range sourceURL.Query() {
		params[key] = strings.Join(val, " ")
	}

	if len(args) == 1 {
		parsed, err := url.Parse(args[0])
		if err != nil {
			return nil, "", err
		}

		if parsed.Path != "" {
			p = p + "/" + parsed.Path
		}

		for key, val := range parsed.Query() {
			params[key] = strings.Join(val, " ")
		}
	}
	return params, p, nil
}

func readVault(source *Source, args ...string) (data []byte, err error) {
	if source.vc == nil {
		source.vc, err = vault.New(source.URL)
		if err != nil {
			return nil, err
		}
		err = source.vc.Login()
		if err != nil {
			return nil, err
		}
	}

	params, p, err := parseVaultParams(source.URL, args)
	if err != nil {
		return nil, err
	}

	source.mediaType = jsonMimetype
	switch {
	case len(params) > 0:
		data, err = source.vc.Write(p, params)
	case strings.HasSuffix(p, "/"):
		source.mediaType = jsonArrayMimetype
		data, err = source.vc.List(p)
	default:
		data, err = source.vc.Read(p)
	}
	if err != nil {
		return nil, err
	}

	if len(data) == 0 {
		return nil, errors.Errorf("no value found for path %s", p)
	}

	return data, nil
}
